package config

import "runtime"

var (
	ConstantFold   bool // Calculates the result of operations on constants at compile time.
	Dry            bool // Skips writing the executable to disk.
	FileAlign      int  // FileAlign is the alignment of the sections in the file.
	MemoryAlign    int  // MemoryAlign is the alignment of the sections in memory.
	ShowAssembly   bool // Shows assembly instructions at the end.
	ShowStatistics bool // Shows statistics at the end.
	Sections       bool // Adds section headers to the executable.
	TargetArch     Arch // Target architecture.
	TargetOS       OS   // Target platform.
)

// Reset resets the configuration to its default values.
func Reset() {
	ShowAssembly = false
	ShowStatistics = false
	Sections = false
	Dry = false

	switch runtime.GOARCH {
	case "amd64":
		SetTargetArch(X86)
	case "arm64":
		SetTargetArch(ARM)
	default:
		SetTargetArch(UnknownArch)
	}

	switch runtime.GOOS {
	case "linux":
		TargetOS = Linux
	case "darwin":
		TargetOS = Mac
	case "windows":
		TargetOS = Windows
	default:
		TargetOS = UnknownOS
	}

	Optimize(true)
}

// Optimize enables or disables all the optimizations at once.
func Optimize(enable bool) {
	ConstantFold = enable
}

// SetTargetArch changes the target architecture and updates the alignment.
func SetTargetArch(arch Arch) {
	TargetArch = arch

	if TargetArch == ARM {
		FileAlign = 0x4000
		MemoryAlign = 0x4000
	} else {
		FileAlign = 0x1000
		MemoryAlign = 0x1000
	}
}
