package x86_test

import (
	"testing"

	"git.urbach.dev/cli/q/src/cpu"
	"git.urbach.dev/cli/q/src/x86"
	"git.urbach.dev/go/assert"
)

func TestCallRegister(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Code     []byte
	}{
		{x86.RAX, []byte{0xFF, 0xD0}},
		{x86.RCX, []byte{0xFF, 0xD1}},
		{x86.RDX, []byte{0xFF, 0xD2}},
		{x86.RBX, []byte{0xFF, 0xD3}},
		{x86.RSP, []byte{0xFF, 0xD4}},
		{x86.RBP, []byte{0xFF, 0xD5}},
		{x86.RSI, []byte{0xFF, 0xD6}},
		{x86.RDI, []byte{0xFF, 0xD7}},
		{x86.R8, []byte{0x41, 0xFF, 0xD0}},
		{x86.R9, []byte{0x41, 0xFF, 0xD1}},
		{x86.R10, []byte{0x41, 0xFF, 0xD2}},
		{x86.R11, []byte{0x41, 0xFF, 0xD3}},
		{x86.R12, []byte{0x41, 0xFF, 0xD4}},
		{x86.R13, []byte{0x41, 0xFF, 0xD5}},
		{x86.R14, []byte{0x41, 0xFF, 0xD6}},
		{x86.R15, []byte{0x41, 0xFF, 0xD7}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("call %s", pattern.Register)
		code := x86.CallRegister(nil, pattern.Register)
		assert.DeepEqual(t, code, pattern.Code)
	}
}

func TestCallAtMemory(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Offset   int8
		Code     []byte
	}{
		{x86.RAX, 0, []byte{0xFF, 0x10}},
		{x86.RCX, 0, []byte{0xFF, 0x11}},
		{x86.RDX, 0, []byte{0xFF, 0x12}},
		{x86.RBX, 0, []byte{0xFF, 0x13}},
		{x86.RSP, 0, []byte{0xFF, 0x14, 0x24}},
		{x86.RBP, 0, []byte{0xFF, 0x55, 0x00}},
		{x86.RSI, 0, []byte{0xFF, 0x16}},
		{x86.RDI, 0, []byte{0xFF, 0x17}},
		{x86.R8, 0, []byte{0x41, 0xFF, 0x10}},
		{x86.R9, 0, []byte{0x41, 0xFF, 0x11}},
		{x86.R10, 0, []byte{0x41, 0xFF, 0x12}},
		{x86.R11, 0, []byte{0x41, 0xFF, 0x13}},
		{x86.R12, 0, []byte{0x41, 0xFF, 0x14, 0x24}},
		{x86.R13, 0, []byte{0x41, 0xFF, 0x55, 0x00}},
		{x86.R14, 0, []byte{0x41, 0xFF, 0x16}},
		{x86.R15, 0, []byte{0x41, 0xFF, 0x17}},

		{x86.RAX, 1, []byte{0xFF, 0x50, 0x01}},
		{x86.RCX, 1, []byte{0xFF, 0x51, 0x01}},
		{x86.RDX, 1, []byte{0xFF, 0x52, 0x01}},
		{x86.RBX, 1, []byte{0xFF, 0x53, 0x01}},
		{x86.RSP, 1, []byte{0xFF, 0x54, 0x24, 0x01}},
		{x86.RBP, 1, []byte{0xFF, 0x55, 0x01}},
		{x86.RSI, 1, []byte{0xFF, 0x56, 0x01}},
		{x86.RDI, 1, []byte{0xFF, 0x57, 0x01}},
		{x86.R8, 1, []byte{0x41, 0xFF, 0x50, 0x01}},
		{x86.R9, 1, []byte{0x41, 0xFF, 0x51, 0x01}},
		{x86.R10, 1, []byte{0x41, 0xFF, 0x52, 0x01}},
		{x86.R11, 1, []byte{0x41, 0xFF, 0x53, 0x01}},
		{x86.R12, 1, []byte{0x41, 0xFF, 0x54, 0x24, 0x01}},
		{x86.R13, 1, []byte{0x41, 0xFF, 0x55, 0x01}},
		{x86.R14, 1, []byte{0x41, 0xFF, 0x56, 0x01}},
		{x86.R15, 1, []byte{0x41, 0xFF, 0x57, 0x01}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("call [%s+%d]", pattern.Register, pattern.Offset)
		code := x86.CallAtMemory(nil, pattern.Register, pattern.Offset)
		assert.DeepEqual(t, code, pattern.Code)
	}
}
